// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_FORMATTER_IMPL_DECORATE_HPP
#define BOOST_FORMATTER_IMPL_DECORATE_HPP

namespace boost { namespace io
{
   template< typename CharT >
   class decorate
   {
      public:
         const CharT *                 open;
         const CharT *                 close;
         const CharT *                 separator;
      public:
         typedef io::detail::decoration_on_stream< CharT, 0 >        open_decor;
         typedef io::detail::decoration_on_stream< CharT, 1 >        close_decor;
         typedef io::detail::decoration_on_stream< CharT, 2 >        separator_decor;
      public:
         inline              decorate( const CharT * o, const CharT * c, const CharT * s = 0 ):
            open(      o ),
            close(     c ),
            separator( s )
         {
         }
         inline              decorate( const decorate & d ):
            open(      d.open ),
            close(     d.close ),
            separator( d.separator )
         {
         }
   };

   typedef decorate< char >                                          cdecorate;
   typedef decorate< wchar_t >                                       wdecorate;

   template< typename CharT, class TraitsT >
   inline std::basic_ostream< CharT, TraitsT > & operator<<
   (
      std::basic_ostream< CharT, TraitsT > & os,
      const decorate< CharT >              & decor
   )
   {
      if( decor.open != 0 )            decorate< CharT >::open_decor::set_value(      os, decor.open );
      if( decor.close != 0 )           decorate< CharT >::close_decor::set_value(     os, decor.close );
      if( decor.separator != 0 )       decorate< CharT >::separator_decor::set_value( os, decor.separator );
      return( os );
   }
}}

#endif
