// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_FORMATTER_IMPL_DECORATION_HPP
#define BOOST_FORMATTER_IMPL_DECORATION_HPP

#include <string>

namespace boost { namespace io
{
   template< typename CharT, int id = 0 >
   class decoration
   {
      public:
         typedef CharT                                               char_type;
         typedef std::basic_string< CharT >                          value_type;
         typedef detail::decoration_on_stream< CharT, id >           default_type;
      private:
         value_type                    value;
         int                           default_id;
      public:
         inline const value_type  operator()() const
         {
            return( value );
         }
         template< typename InputStream >
         inline const value_type  operator()( InputStream & is ) const
         {
            return( value == value_type() ? default_type::get_value( is ) : value );
         }
      public:
         inline decoration &      operator=( const value_type v )
         {
            value = v;
            return( *this );
         }
         inline decoration &      operator=( const decoration & d )
         {
            value = d.value;
            return( *this );
         }
      public:
         inline              decoration( const value_type v = value_type()):
            value( v )
         {
         }
         inline              decoration( const decoration & d ):
            value( d.value )
         {
         }
   };
}}

#endif
