// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_FORMATTER_IMPL_DECORATION_ON_STREAM_HPP
#define BOOST_FORMATTER_IMPL_DECORATION_ON_STREAM_HPP

#include <ios>

namespace boost { namespace io { namespace detail
{
   // default_decoration -- default values when there are none provided on the stream

   template< typename CharT, int id > struct default_decoration{};

   template<> struct default_decoration< char, 0 >{    static const char    * value(){ return( "[ "  ); }};
   template<> struct default_decoration< char, 1 >{    static const char    * value(){ return( " ]"  ); }};
   template<> struct default_decoration< char, 2 >{    static const char    * value(){ return( ", "  ); }};

   template<> struct default_decoration< wchar_t, 0 >{ static const wchar_t * value(){ return( L"[ " ); }};
   template<> struct default_decoration< wchar_t, 1 >{ static const wchar_t * value(){ return( L" ]" ); }};
   template<> struct default_decoration< wchar_t, 2 >{ static const wchar_t * value(){ return( L", " ); }};

   // decoration_on_stream -- manage the decoration associated with a stream

   template< typename CharT, int id >
   class decoration_on_stream
   {
      public:
         typedef CharT                                               char_type;
         typedef std::basic_string< CharT >                          value_type;
      public:
         static const int                        get_id()
         {
            static const int           id_ = std::ios::xalloc();
            return( id_ );
         }
      public:
         template< class CharTraits >
         static const CharT *                    get_value( std::basic_ios< CharT, CharTraits >& i )
         {
            const CharT * p = static_cast< const CharT * >( i.pword( get_id()));
            return( p == 0 ? default_decoration< CharT, id >::value() : p );
         }
         template< class CharTraits >
         static void                             set_value( std::basic_ios< CharT, CharTraits >& i, const CharT * v )
         {
            i.pword( get_id()) = const_cast< CharT * >( v );
         }
   };
}}}

#endif
