// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_FORMATTER_IMPL_SEQUENCE_DECORATORS_HPP
#define BOOST_FORMATTER_IMPL_SEQUENCE_DECORATORS_HPP

#include <boost/formatter/impl/wrapper_decorators.hpp>

namespace boost { namespace io
{
   namespace detail
   {
      template< typename CharT, class RetType >
      class sequence_decorators: public detail::wrapper_decorators< CharT, RetType >
      {
         private:
            typedef detail::wrapper_decorators< CharT, RetType >     base_type;
         public:
            decoration< CharT, 2 >     separator;
         public:
            inline RetType &                     decorate( const value_type & s )
            {
               separator = s;
               return( *static_cast< RetType * >( this ));
            }
            inline RetType &                     decorate( const value_type & o, const value_type & c )
            {
               base_type::decorate( o, c );
               return( *static_cast< RetType * >( this ));
            }
            inline RetType &                     decorate( const value_type & o, const value_type & c, const value_type & s )
            {
               base_type::decorate( o, c );
               separator = s;
               return( *static_cast< RetType * >( this ));
            }
            template< typename RT >
            inline RetType &                     decorate( const wrapper_decorators< CharT, RT > & wd )
            {
               return( base_type::decorate( wd ));
            }
            template< typename RT >
            inline RetType &                     decorate( const wrapper_decorators< CharT, RT > & wd, const value_type & s )
            {
               separator = s;
               return( base_type::decorate( wd ));
            }
            template< typename RT >
            inline RetType &                     decorate( const sequence_decorators< CharT, RT > & sd )
            {
               base_type::decorate( sd );
               separator = sd.separator;
               return( *static_cast< RetType * >( this ));
            }
         public:
            inline           sequence_decorators( const value_type & s = value_type()):
               separator( s )
            {
            }
            inline           sequence_decorators( const value_type & o, const value_type & c, const value_type & s = value_type()):
               base_type( o, c ),
               separator( s )
            {
            }
            template< typename RT >
            inline           sequence_decorators( const detail::wrapper_decorators< CharT, RT > & wd, const value_type & s = value_type()):
               base_type( wd ),
               separator( s )
            {
            }
            template< typename RT >
            inline           sequence_decorators( const detail::sequence_decorators< CharT, RT > & sd ):
               base_type( sd ),
               separator( sd.separator )
            {
            }
      };
   }

   template< typename CharT >
   class sequence_decorators: public detail::sequence_decorators< CharT, sequence_decorators< CharT > >
   {
      private:
         typedef detail::sequence_decorators< CharT, boost::io::sequence_decorators< CharT > >
                                                                     base_type;
      public:
         inline              sequence_decorators( const value_type & s = value_type()):
            base_type( s )
         {
         }
         inline              sequence_decorators( const value_type & o, const value_type & c, const value_type & s = value_type()):
            base_type( o, c, s )
         {
         }
         template< typename RT >
         inline              sequence_decorators( const detail::wrapper_decorators< CharT, RT > & wd, const value_type & s = value_type()):
            base_type( wd, s )
         {
         }
         template< typename RT >
         inline              sequence_decorators( const detail::sequence_decorators< CharT, RT > & sd ):
            base_type( sd )
         {
         }
   };
}}

#endif
