// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_FORMATTER_IMPL_WRAPPER_DECORATORS_HPP
#define BOOST_FORMATTER_IMPL_WRAPPER_DECORATORS_HPP

#include <boost/formatter/impl/decoration.hpp>

namespace boost { namespace io
{
   namespace detail
   {
      template< typename CharT, class RetType >
      class wrapper_decorators
      {
         public:
            typedef typename decoration< CharT >::char_type          char_type;
            typedef typename decoration< CharT >::value_type         value_type;
         public:
            decoration< CharT, 0 >     open;
            decoration< CharT, 1 >     close;
         public:
            inline RetType &                     decorate( const value_type & o, const value_type & c )
            {
               open  = o;
               close = c;
               return( *static_cast< RetType * >( this ));
            }
            template< class RT >
            inline RetType &                     decorate( const wrapper_decorators< CharT, RT > & wd )
            {
               open  = wd.open;
               close = wd.close;
               return( *static_cast< RetType * >( this ));
            }
         public:
            inline           wrapper_decorators()
            {
            }
            template< class RT >
            inline           wrapper_decorators( const wrapper_decorators< CharT, RT > & wd ):
               open( wd.open ), close( wd.close )
            {
            }
            inline           wrapper_decorators( const value_type & o, const value_type & c ):
               open( o ), close( c )
            {
            }
      };
   }

   template< typename CharT >
   class wrapper_decorators: public detail::wrapper_decorators< CharT, wrapper_decorators< CharT > >
   {
      private:
         typedef detail::wrapper_decorators< CharT, boost::io::wrapper_decorators< CharT > >
                                                                     base_type;
      public:
         inline              wrapper_decorators()
         {
         }
         inline              wrapper_decorators( const value_type & o, const value_type & c ):
            base_type( o, c )
         {
         }
         template< typename RT >
         inline              wrapper_decorators( const detail::wrapper_decorators< CharT, RT > & wd ):
            base_type( wd )
         {
         }
   };
}}

#endif
