// (C) Copyright 2004: Reece H. Dunn
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <boost/formatter/decorators.hpp>
#include <boost/test/minimal.hpp>

namespace io = boost::io;

int test_main( int, char *[] )
{
   io::wrapper_decorators< char >      wrap( "{ ", " }" );

   io::sequence_decorators< char >     seq1;
   BOOST_TEST( seq1.open()  == std::string());
   BOOST_TEST( seq1.close() == std::string());
   BOOST_TEST( seq1.separator() == std::string());

   io::sequence_decorators< char >     seq2( " : " );
   BOOST_TEST( seq2.open()  == std::string());
   BOOST_TEST( seq2.close() == std::string());
   BOOST_TEST( seq2.separator() == " : " );

   io::sequence_decorators< char >     seq3( "( ", " )" );
   BOOST_TEST( seq3.open()  == "( " );
   BOOST_TEST( seq3.close() == " )" );
   BOOST_TEST( seq3.separator() == std::string());

   io::sequence_decorators< char >     seq4( "[ ", " ]", " | " );
   BOOST_TEST( seq4.open()  == "[ " );
   BOOST_TEST( seq4.close() == " ]" );
   BOOST_TEST( seq4.separator() == " | " );

   io::sequence_decorators< char >     seq5( wrap );
   BOOST_TEST( seq5.open()  == "{ " );
   BOOST_TEST( seq5.close() == " }" );
   BOOST_TEST( seq5.separator() == std::string());

   io::sequence_decorators< char >     seq6( wrap, " : " );
   BOOST_TEST( seq6.open()  == "{ " );
   BOOST_TEST( seq6.close() == " }" );
   BOOST_TEST( seq6.separator() == " : " );

   io::sequence_decorators< char >     seq7( seq3, " : " );
   BOOST_TEST( seq7.open()  == "( " );
   BOOST_TEST( seq7.close() == " )" );
   BOOST_TEST( seq7.separator() == " : " );

   io::sequence_decorators< char >     seq8( seq4 );
   BOOST_TEST( seq8.open()  == "[ " );
   BOOST_TEST( seq8.close() == " ]" );
   BOOST_TEST( seq8.separator() == " | " );

   seq1.decorate( " - " );
   BOOST_TEST( seq1.open()  == std::string());
   BOOST_TEST( seq1.close() == std::string());
   BOOST_TEST( seq1.separator() == " - " );

   seq1.decorate( "[ ", " ]" );
   BOOST_TEST( seq1.open()  == "[ " );
   BOOST_TEST( seq1.close() == " ]" );
   BOOST_TEST( seq1.separator() == " - " );

   seq1.decorate( "<< ", " >>", ", " );
   BOOST_TEST( seq1.open()  == "<< " );
   BOOST_TEST( seq1.close() == " >>" );
   BOOST_TEST( seq1.separator() == ", " );

   seq1.decorate( wrap );
   BOOST_TEST( seq1.open()  == "{ " );
   BOOST_TEST( seq1.close() == " }" );
   BOOST_TEST( seq1.separator() == ", " );

   seq1.decorate( wrap, " : " );
   BOOST_TEST( seq1.open()  == "{ " );
   BOOST_TEST( seq1.close() == " }" );
   BOOST_TEST( seq1.separator() == " : " );

   seq1.decorate( wrap, " - " );
   BOOST_TEST( seq1.open()  == "{ " );
   BOOST_TEST( seq1.close() == " }" );
   BOOST_TEST( seq1.separator() == " - " );

   seq1.decorate( seq3, " : " );
   BOOST_TEST( seq1.open()  == "( " );
   BOOST_TEST( seq1.close() == " )" );
   BOOST_TEST( seq1.separator() == " : " );

   seq1.decorate( seq4 );
   BOOST_TEST( seq1.open()  == "[ " );
   BOOST_TEST( seq1.close() == " ]" );
   BOOST_TEST( seq1.separator() == " | " );

   seq1.decorate( "<-- ", " -->" ).decorate( " - " );
   BOOST_TEST( seq1.open()  == "<-- " );
   BOOST_TEST( seq1.close() == " -->" );
   BOOST_TEST( seq1.separator() == " - " );

   return( 0 );
}
